#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <signal.h>

#include "applet.h"

/*
 * Domain status masks
 */
#define vm_state2mask(_s)    (1 << ((int)_s))

typedef enum {
    VM_SMASK_NONE       = 0,
    VM_SMASK_RUNNING    = vm_state2mask(VIR_DOMAIN_RUNNING),
    VM_SMASK_BLOCKED    = vm_state2mask(VIR_DOMAIN_BLOCKED),
    VM_SMASK_PAUSED     = vm_state2mask(VIR_DOMAIN_PAUSED),
    VM_SMASK_SHUTDOWN   = vm_state2mask(VIR_DOMAIN_SHUTDOWN),
    VM_SMASK_SHUTOFF    = vm_state2mask(VIR_DOMAIN_SHUTOFF),

    VM_SMASK_ALL        = (VM_SMASK_RUNNING | VM_SMASK_BLOCKED |
                           VM_SMASK_PAUSED | VM_SMASK_SHUTDOWN |
                           VM_SMASK_SHUTOFF )
} VmAppetSmask;

#define vm_state_is_valid(_s)    (vm_state2mask(_s) & VM_SMASK_ALL)

/*
 * Action menu rows
 */
typedef struct VmAppletActmenuRow
{
    const char      *label;     /* menu item label */
    const char      *stock_id;  /* menu item icon */
    gboolean        hide;       /* hide the item instead to use insensitive mode */

    void            (*callback)(GtkMenuItem *, gpointer);   /* callback function */

    VmAppetSmask    mask;       /* set the item sensitive/insensitive when
                                   this mask match/mismatch with domain status */

    VmAppetSmask    imask;      /* always set the item insensitive
                                   (idenpendetly on the 'hide' setting) */
} VmAppletActmenuRow;

static void vm_amenu_edit_cb(GtkMenuItem *menuitem, gpointer data);
static void vm_amenu_run_cb(GtkMenuItem *menuitem, gpointer data);
static void vm_amenu_pause_cb(GtkMenuItem *menuitem, gpointer data);
static void vm_amenu_resume_cb(GtkMenuItem *menuitem, gpointer data);
static void vm_amenu_shutdown_cb(GtkMenuItem *menuitem, gpointer data);
static void vm_amenu_console_cb(GtkMenuItem *menuitem, gpointer data);

/* Applet actions menu rows */
static VmAppletActmenuRow vm_amenu_rows[] =
{
    { "Details...", VM_STOCK_DO_DETAILS,  0, vm_amenu_edit_cb,     VM_SMASK_ALL, 0 },

#ifdef ENABLE_CONSOLEHELPER
    { NULL, NULL, 0, NULL, 0, 0 }, /* separator */

    { N_("Run"),        VM_STOCK_DO_RUN,      0, vm_amenu_run_cb,      VM_SMASK_SHUTOFF, 0 },
    { N_("Pause"),      VM_STOCK_DO_PAUSE,    1, vm_amenu_pause_cb,    VM_SMASK_RUNNING | VM_SMASK_BLOCKED, VM_SMASK_SHUTOFF },
    { N_("Resume"),     VM_STOCK_DO_RESUME,   1, vm_amenu_resume_cb,   VM_SMASK_PAUSED, 0 },
    { N_("Shutdown"),   VM_STOCK_DO_SHUTDOWN, 0, vm_amenu_shutdown_cb, VM_SMASK_RUNNING | VM_SMASK_BLOCKED, 0 },

    { NULL, NULL, 0, NULL, 0, 0 }, /* separator */
#endif

    { N_("Open..."),    VM_STOCK_DO_OPEN,     0, vm_amenu_console_cb,  VM_SMASK_RUNNING | VM_SMASK_BLOCKED },
};


static void
vm_amenu_action_exec(VmAppletDomain *domain, char *action)
{
    pid_t pid;

    _F();

    if (action==NULL || *action=='\0' || domain==NULL)
        return;

    signal(SIGCHLD, vm_catch_child);

    _D("exec-> %s\n", action);

    switch((pid=fork()))
    {
        case -1:    /* error */
            return;
        case 0:     /* child */
            execlp(VM_WRAPPER_NAME, VM_WRAPPER_NAME, action,
                        domain->conn->uri_str, domain->uuid, NULL);
            exit(EXIT_FAILURE);
        default: /* parent */
            break;
    }
}

#ifdef ENABLE_CONSOLEHELPER
static void
vm_amenu_pause_cb(GtkMenuItem *menuitem, gpointer data)
{
    vm_amenu_action_exec(((VmAppletCtl *) data)->actdom, "suspend");
}
static void
vm_amenu_resume_cb(GtkMenuItem *menuitem, gpointer data)
{
    vm_amenu_action_exec(((VmAppletCtl *) data)->actdom, "resume");
}

static void
vm_amenu_shutdown_cb(GtkMenuItem *menuitem, gpointer data)
{
    vm_amenu_action_exec(((VmAppletCtl *) data)->actdom, "shutdown");
}
static void
vm_amenu_run_cb(GtkMenuItem *menuitem, gpointer data)
{
    vm_amenu_action_exec(((VmAppletCtl *) data)->actdom, "create");
}
#endif /* ENABLE_CONSOLEHELPER */

/* deactivate also domains menu */
static void
vm_amenu_deactivate_cb(GtkMenuShell *menushell, VmAppletCtl *ctl)
{
    _F();
    gtk_menu_popdown(GTK_MENU(ctl->popmenu));
}

static void
vm_amenu_edit_cb(GtkMenuItem *menuitem, gpointer data)
{
    _F();
    vm_mngr_exec(((VmAppletCtl *) data)->actdom, "--show-domain-editor");
}

static void
vm_amenu_console_cb(GtkMenuItem *menuitem, gpointer data)
{
    _F();
    vm_mngr_exec(((VmAppletCtl *) data)->actdom, "--show-domain-console");
}


static GtkWidget *
vm_amenu_new(gpointer data)
{
    GtkWidget *menu;
    int i;

    _F();

    menu = gtk_menu_new();

    for (i=0; i < G_N_ELEMENTS(vm_amenu_rows); i++)
    {
        VmAppletActmenuRow *row = &vm_amenu_rows[i];
        GtkWidget *item;

        if (row->label)
        {
            item = gtk_image_menu_item_new_with_label(_(row->label));

            if (row->stock_id)
            {
                GtkWidget *image = gtk_image_new_from_stock (row->stock_id, GTK_ICON_SIZE_MENU);
                gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
            }
            gtk_widget_set_sensitive(item, TRUE);
            g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(row->callback), data);
        }
        else
            item = gtk_separator_menu_item_new();

        g_object_set_data(G_OBJECT(item), "vmrow", (gpointer) row);
        gtk_container_add(GTK_CONTAINER(menu), item);
    }

    g_signal_connect(G_OBJECT(menu), "deactivate",
                    G_CALLBACK(vm_amenu_deactivate_cb), data);

    gtk_widget_show_all(menu);
    return menu;
}

/*
 * The item sensitivity depends on the selected domain state.
 * We don't offer all actions for all domains. The setting is
 * done by VmAppletActmenuRow->mask (with exception for Xen
 * dom0).
 */
static void
vm_amenu_item_sensitive(GtkWidget *item, gpointer data)
{
    VmAppletDomain *domain = (VmAppletDomain *) data;
    VmAppletActmenuRow *row;

    _F();

    row = (VmAppletActmenuRow *) g_object_get_data(G_OBJECT(item), "vmrow");

    if (!row->label)
        return; /* menu separator */

    if (!vm_state_is_valid(domain->activity))
        /* invalid state */
        gtk_widget_set_sensitive(item, FALSE);
    else if (row->mask & vm_state2mask(domain->activity))
    {
        /* set the item sensitive */
        if (row->hide)
            gtk_widget_show(item);
        gtk_widget_set_sensitive(item, TRUE);
    }
    else if (row->imask & vm_state2mask(domain->activity))
    {
        /* set the item insensitive */
        if (row->hide)
            gtk_widget_show(item);
        gtk_widget_set_sensitive(item, FALSE);
    }
    else
    {
        /* set the item insensitive/hidden */
        if (row->hide)
            gtk_widget_hide(item);
        else
            gtk_widget_set_sensitive(item, FALSE);
    }
}

gboolean
vm_menuitem_cb(GtkWidget *menuitem, GdkEventButton *event, gpointer data)

{
    VmAppletDomain *domain = (VmAppletDomain *) data;

    _F();

    if (domain==NULL || domain->ctl==NULL)
        return TRUE;

    if (!domain->ctl->actmenu)
        domain->ctl->actmenu = vm_amenu_new(domain->ctl);

    domain->ctl->actdom = domain;

    /* disable some items fro dom0 */
    gtk_container_foreach(GTK_CONTAINER(domain->ctl->actmenu),
            vm_amenu_item_sensitive, domain);

    gtk_menu_popup (GTK_MENU (domain->ctl->actmenu),
            NULL, NULL, NULL, NULL, event->button, event->time);

    return TRUE;
}

/*
 * vim: set tabstop=4:
 * vim: set shiftwidth=4:
 * vim: set expandtab:
 */
