#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gconf/gconf-client.h>
#include <panel-applet-gconf.h>

#include "applet.h"

typedef struct VmconnDialog
{
    VmAppletConn    *conn;      /* selected/new connection */
    VmAppletCtl     *ctl;       /* applet control struct */
    GtkWidget       *d_conns;   /* dialog with list of connections */
    GtkWidget       *tv_conns;  /* treeview with connections */
    GtkWidget       *f_remote;  /* frame with remote options */
    GtkWidget       *d_edit;    /* add/edit dialog */
    GSList          *elems;     /* d_edit widgets relevant for URI */
} VmconnDialog;

static VmAppletOption vmconn_drivers[] =
{
        { "xen",    N_("Xen") },
        { "qemu",   N_("Qemu & KVM") },
        { "test",   N_("Test") }
};

static VmAppletOption vmconn_transports[] =
{
        { NULL,    NULL },
        { "tls",   N_("TLS (SSL)") },
        { "unix",  N_("Unix domain socket") },
        { "ssh",   N_("SSH") },
        { "ext",   N_("External program") },
        { "tcp",   N_("Unencrypted TCP/IP socket") }
};

/* dialog responses */
enum {
    VMCONN_RESPONSE_ADD = 1,
    VMCONN_RESPONSE_EDIT,
    VMCONN_RESPONSE_DELETE
};

/* list store columns */
enum {
    VMCONN_COL_ENABLED,
    VMCONN_COL_NAME,
    VMCONN_COL_URI,
    VMCONN_COL_STATUS,
    VMCONN_COL_CONNDATA,
    VMCONN_COL_DLGDATA,

    N_VMCONN_COLS
};

static void vmconn_append_conn(VmconnDialog *dlg, VmAppletConn *conn, gboolean active);
static void vmconn_refresh_conn(VmconnDialog *dlg, VmAppletConn *conn);
static void vmconn_remove_conn(VmconnDialog *dlg, VmAppletConn *conn);

char *
vm_connection_mk_name(xmlURIPtr uri)
{
    return g_strdup_printf("%s [%s]", uri->server ? : "localhost", uri->scheme);
}

VmAppletConn *
vm_connection_new(const char *uri)
{
    VmAppletConn *conn;

    _F();

    _D("allocate %s", uri);
    conn = g_new0(VmAppletConn, 1);
    if (!conn)
       return NULL;

    if (uri) {
        conn->uri = xmlParseURI(uri);
        if (!conn->uri) {
            _D("parse %s failed", uri);
            vm_connection_free(conn);
            return NULL;
        }
        conn->uri_str = g_strdup(uri);
        conn->name = vm_connection_mk_name(conn->uri);
    }
    else
        conn->uri = xmlCreateURI();

    return conn;
}

void
vm_connection_free(VmAppletConn *conn)
{
    _F();

    g_free(conn->name);
    g_free(conn->uri_str);
    if (conn->uri)
        xmlFreeURI(conn->uri);
    g_free(conn);
}

static int
vm_connection_refresh(VmAppletCtl *ctl, VmAppletConn *conn)
{
    _F();

    if (!conn->enabled)
        return TRUE;

    if (!conn->ptr)
    {
        _D("connectiong to %s", conn->uri_str);
        conn->ptr = virConnectOpenReadOnly(conn->uri_str);
        if (conn->ptr) {
            virNodeGetInfo(conn->ptr, &conn->ninfo);
            ctl->naconns++;
        }
    }

    return conn->ptr ? TRUE : FALSE;
}

/* note, the connection has to be already added to the list of connections */
void
vm_connection_enable(VmAppletCtl *ctl, VmAppletConn *conn, gboolean status)
{
    gboolean old;

    _F();
    _D("%s (%s): %s", conn->name,
            conn->enabled ? "enabled" : "disabled",
            status ? "enabled" : "disabled");

    old = conn->enabled;
    conn->enabled = status;

    if (old && status == FALSE)
    {
        _D("disabling %s", conn->uri_str);
        if (conn->dom)
            vm_domainslist_free(conn->dom);
        if (conn->ptr) {
            virConnectClose(conn->ptr);
            ctl->naconns--;
        }
        conn->ptr = NULL;
        conn->dom = NULL;
        ctl->neconns--;
    }
    else if (old == FALSE && status) {
        _D("enabling: %s", conn->uri_str);
        vm_connection_refresh(ctl, conn);
        ctl->neconns++;
    }
}

int
vm_connections_refresh(VmAppletCtl *ctl)
{
    VmAppletConn *conn = ctl->conn;

    _F();

    while(conn)
    {
        vm_connection_refresh(ctl, conn);
        conn = conn->next;
    }
    _D("enable/active connections: %d/%d", ctl->neconns, ctl->naconns);
    return ctl->naconns ? TRUE : FALSE;
}

void
vm_connections_destroy(VmAppletCtl *ctl)
{
    VmAppletConn *conn = ctl->conn;

    _F();

    while(conn)
    {
        VmAppletConn *next =  conn->next;

        vm_connection_enable(ctl, conn, FALSE);
        vm_connection_free(conn);

        conn = next;
    }

    ctl->conn = NULL;
    ctl->nconns = 0;
    ctl->naconns = 0;
    ctl->neconns = 0;
}

void
vm_connections_gconf_write(VmAppletCtl *ctl)
{
    VmAppletConn *conn = ctl->conn;
    GSList *a_list = NULL, *i_list = NULL;

    while(conn)
    {
        if (conn->enabled)
            a_list = g_slist_append(a_list, conn->uri_str);
        else
            i_list = g_slist_append(i_list, conn->uri_str);

        conn = conn->next;
    }

    /* active list */
    panel_applet_gconf_set_list(PANEL_APPLET(ctl->applet), VM_GCONF_ACTIVE_URI,
                            GCONF_VALUE_STRING, a_list, NULL);
    /* inactive list */
    panel_applet_gconf_set_list(PANEL_APPLET(ctl->applet), VM_GCONF_INACTIVE_URI,
                            GCONF_VALUE_STRING, i_list, NULL);

    g_slist_free(a_list);
    g_slist_free(i_list);
}

static void
vm_connections_from_list(VmAppletCtl *ctl, GSList *list, gboolean enabled)
{
    _F();

    while(list) {
        _D("list item: %s", list->data);
        if (list->data) {
            VmAppletConn *conn = vm_connection_new(list->data);
            if (conn) {
                vm_connection_add(ctl, conn);
                vm_connection_enable(ctl, conn, enabled);
            }
        }
        list = list->next;
    }
}

void
vm_connections_gconf_read(VmAppletCtl *ctl)
{
    GSList *list;

    _F();

    /* active connections */
    list = panel_applet_gconf_get_list(PANEL_APPLET(ctl->applet),
                            VM_GCONF_ACTIVE_URI,
                            GCONF_VALUE_STRING,
                            NULL);
    if (list) {
        _D("add active connections");
        vm_connections_from_list(ctl, list, TRUE);
        g_slist_free(list);
    }

    /* inactive connections */
    list = panel_applet_gconf_get_list(PANEL_APPLET(ctl->applet),
                            VM_GCONF_INACTIVE_URI,
                            GCONF_VALUE_STRING,
                            NULL);
    if (list) {
        _D("add inactive connections");
        vm_connections_from_list(ctl, list, FALSE);
        g_slist_free(list);
    }
}

VmAppletConn *
vm_connection_add(VmAppletCtl *ctl, VmAppletConn *conn)
{
    VmAppletConn *c = ctl->conn;

    _F();
    _D("adding %s", conn->uri_str);

    while(c && c->next)
        c = c->next;
    if (!c)
        ctl->conn = conn;
    else
        c->next = conn;

    ctl->nconns++;

    return conn;
}

static void
vm_connection_remove(VmAppletCtl *ctl, VmAppletConn *conn)
{
    VmAppletConn *pre = NULL, *c = ctl->conn;

    _F();
    _D("removing %s", conn->uri_str);

    while(c) {
       if (c == conn) {
            if (pre)
                pre->next = conn->next;
            else
                ctl->conn = conn->next;
            ctl->nconns--;
            break;
       }
       pre = conn;
       conn = conn->next;
    }
}

static void
vmconn_uri_widget_add(VmconnDialog *dlg, const char *name, GtkWidget *widget)
{
    _F();

    gtk_widget_set_name(widget, name);
    dlg->elems = g_slist_prepend(dlg->elems, widget);
}

static GtkWidget *
vmconn_uri_widget_get(VmconnDialog *dlg, const char *name)
{
    GSList *list;

    _F();

    for(list = dlg->elems; list; list = g_slist_next(list))
    {
        GtkWidget *w = GTK_WIDGET(list->data);
        if (strcmp(gtk_widget_get_name(w), name) == 0)
            return w;
    }
    return NULL;
}

static void
vmconn_is_remote_cb(GtkWidget *widget, VmconnDialog *dlg)
{
    gint tmp = GTK_TOGGLE_BUTTON(widget)->active;
    _F();
    gtk_widget_set_sensitive(dlg->f_remote, tmp);
}

static void
vmconn_set_str(const char *name, char **old, const char *new)
{
    _F();

    if (new && *new && (*old == NULL || strcmp(*old, new))) {
        _D("%s: %s --> %s", name, *old, new);
        g_free(*old);
        *old = g_strdup(new);
    }
    else if ((new == NULL || *new == '\0') && *old) {
        _D("%s: zeroize", name);
        g_free(*old);
        *old = NULL;
    }
}

static void
vmconn_edit_response_cb(GtkWidget *widget, gint id,  VmconnDialog *dlg)
{
    VmAppletConn *conn;
    xmlURIPtr uri;
    GtkTreeIter iter;
    GtkWidget *w;
    const gchar *val = NULL;

    _F();

    if (id != GTK_RESPONSE_CLOSE)
        return;

    conn = dlg->conn ? : vm_connection_new(NULL);
    uri = conn->uri;

    w = vmconn_uri_widget_get(dlg, "driver");
    if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(w), &iter))
	    gtk_tree_model_get(gtk_combo_box_get_model(
                        GTK_COMBO_BOX(w)), &iter, 1, &val, -1);
    vmconn_set_str("driver", &uri->scheme, val);
    val = NULL;
 
    w = vmconn_uri_widget_get(dlg, "remote");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
        w = vmconn_uri_widget_get(dlg, "transport");
        if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(w), &iter))
            gtk_tree_model_get(gtk_combo_box_get_model(
                            GTK_COMBO_BOX(w)), &iter, 1, &val, -1);
        if (val && *val) {
            gchar *nscheme = g_strdup_printf("%s+%s", uri->scheme, val);
            g_free(uri->scheme);
            uri->scheme = nscheme;
        }
        val = NULL;

        w = vmconn_uri_widget_get(dlg, "host");
        val = gtk_entry_get_text(GTK_ENTRY(w));
        vmconn_set_str("host", &uri->server, val);

        w = vmconn_uri_widget_get(dlg, "username");
        val = gtk_entry_get_text(GTK_ENTRY(w));
        vmconn_set_str("username", &uri->user, val);

        w = vmconn_uri_widget_get(dlg, "port");
        val = gtk_entry_get_text(GTK_ENTRY(w));
        uri->port = val ? atoi(val) : 0;
    } else {
        g_free(uri->server);
        g_free(uri->user);
        uri->server = uri->user = NULL;
        uri->port = 0;
    }

    w = vmconn_uri_widget_get(dlg, "params");
    val = gtk_entry_get_text(GTK_ENTRY(w));
    vmconn_set_str("params", &uri->query, val);
    if (uri->query_raw) {
        g_free(uri->query_raw);
        uri->query_raw = NULL;
    }

    w = vmconn_uri_widget_get(dlg, "path");
    val = gtk_entry_get_text(GTK_ENTRY(w));
    vmconn_set_str("path", &uri->path, val);

    if (conn->uri_str)
        g_free(conn->uri_str);

    conn->uri_str = (char *) xmlSaveUri(uri);
    _D("new URI: %s", conn->uri_str);

    if (!conn->uri_str) {
        vm_error_dialog(dlg->ctl->applet, _("Failed to convert your setting to URI"));
        if (!dlg->conn)
            vm_connection_free(conn);
        return;
    }

    if (dlg->elems)
        g_slist_free(dlg->elems);
    dlg->elems = NULL;
    gtk_widget_destroy(dlg->d_edit);
    dlg->d_edit = NULL;

    if (conn->name)
        g_free(conn->name);
    conn->name = vm_connection_mk_name(conn->uri);

    if (!dlg->conn) {
        /* new connection -- added, but not enabled */
        dlg->conn = conn;
        vmconn_append_conn(dlg, conn, TRUE);
        vm_connection_add(dlg->ctl, conn);
    }
    else {
        vmconn_refresh_conn(dlg, conn);

        /* reset the connection */
        if (conn->enabled) {
            vm_connection_enable(dlg->ctl, conn, FALSE);
            vm_connection_enable(dlg->ctl, conn, TRUE);
        }
    }
}

/*
 * creates dialog for URI definition:
 */
static void
vnconn_edit_dialog(VmconnDialog *dlg)
{
    xmlURIPtr uri = dlg->conn ? dlg->conn->uri : NULL;
    GtkWidget *win;
    GtkWidget *frame;
    GtkWidget *widget;
    GtkWidget *vbox;
    GtkSizeGroup *sgrp;
    char *driver = NULL, *trans = NULL, *port = NULL;

    _F();
    _D(uri ? "modify a connection" : "define a new connection");

    if (uri && uri->scheme) {
        driver = g_strdup(uri->scheme);

        if ((trans = strchr(driver, '+')) && *(trans+1)) {
           *trans = '\0';
            trans++;
        }
        _D("scheme: %s, driver: %s, trans: %s", uri->scheme, driver, trans);
    }

    if (uri && uri->port)
        port = g_strdup_printf("%d", uri->port);

    win = gtk_dialog_new_with_buttons(
                dlg->conn ? _("Edit Connection") : _("Add Connection"),
                NULL,
                GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
                GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                NULL);

    dlg->d_edit = win;
    vbox = GTK_DIALOG(win)->vbox;
    gtk_box_set_spacing(GTK_BOX(vbox), VM_APPLET_BOX_SPACE);

    if (dlg->ctl->applet)
        gtk_window_set_screen(GTK_WINDOW(win),
                    gtk_widget_get_screen(dlg->ctl->applet));

    gtk_dialog_set_default_response(GTK_DIALOG(win), GTK_RESPONSE_CLOSE);
    gtk_dialog_set_has_separator(GTK_DIALOG(win), FALSE);
    gtk_container_set_border_width(GTK_CONTAINER(win), 12);
    gtk_window_set_resizable(GTK_WINDOW(win), TRUE);
    gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER);
    gtk_window_set_default_size(GTK_WINDOW(win), 300, 400);

    g_signal_connect(G_OBJECT(win), "response",
            G_CALLBACK(vmconn_edit_response_cb), dlg);

    sgrp = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

    frame = vm_frame_new(vbox, _("Connection type"));

    /* Driver */
    widget = vm_combobox_new(vmconn_drivers,
                ARRAY_SIZE(vmconn_drivers), driver);
    vm_prefbox_add(frame, sgrp, _("Driver:"), widget, NULL);
    vmconn_uri_widget_add(dlg, "driver", widget);

    /* Local host */
    widget = vm_checkbutton_new(_("Remote connection"),
                uri && uri->server ? TRUE : FALSE );
    g_signal_connect (G_OBJECT(widget), "toggled",
                    G_CALLBACK(vmconn_is_remote_cb), (gpointer) dlg);
    gtk_box_pack_start(GTK_BOX(frame), widget, FALSE, FALSE, 0);
    vmconn_uri_widget_add(dlg, "remote", widget);

    frame = vm_frame_new(vbox, _("Remote Options"));
    dlg->f_remote = frame;
    gtk_widget_set_sensitive(frame, uri && uri->server ? TRUE : FALSE);

    /* Hostname */
    widget = vm_entry_new(uri ? uri->server : NULL);
    vm_prefbox_add(frame, sgrp, _("Host:"), widget, NULL);
    vmconn_uri_widget_add(dlg, "host", widget);

    /* Port */
    widget = vm_entry_new(port);
    vm_prefbox_add(frame, sgrp, _("Port:"), widget, NULL);
    vmconn_uri_widget_add(dlg, "port", widget);

    /* Username */
    widget = vm_entry_new(uri ? uri->user : NULL);
    vm_prefbox_add(frame, sgrp, _("Username:"), widget, NULL);
    vmconn_uri_widget_add(dlg, "username", widget);

    /* Transport */
    widget = vm_combobox_new(vmconn_transports,
                ARRAY_SIZE(vmconn_transports), trans);
    vm_prefbox_add(frame, sgrp, _("Transport:"), widget, NULL);
    vmconn_uri_widget_add(dlg, "transport", widget);


    frame = vm_frame_new(vbox, _("Advanced Options"));

    /* Path */
    widget = vm_entry_new(uri ? uri->path : NULL);
    vm_prefbox_add(frame, sgrp, _("Path:"), widget, NULL);
    vmconn_uri_widget_add(dlg, "path", widget);

    /* Extra parameters */
    widget = vm_entry_new(uri ? uri->query : NULL);
    vm_prefbox_add(frame, sgrp, _("Parameters:"), widget, NULL);
    vmconn_uri_widget_add(dlg, "params", widget);

    if (uri) {
        g_free(driver);
        g_free(port);
    }

    gtk_widget_show_all(win);
}

static void
vmconn_dialog_quit(VmconnDialog *dlg)
{
    if (dlg->elems)
        g_slist_free(dlg->elems);
    if (dlg->d_edit)
        gtk_widget_destroy(dlg->d_edit);
    if (dlg->d_conns)
        gtk_widget_destroy(dlg->d_conns);

    vm_connections_gconf_write(dlg->ctl);
    free(dlg);
}

static void
vmconn_response_cb(GtkWidget *widget, gint id,  VmconnDialog *dlg)
{
    _F();

    dlg->conn = NULL;

    if (id == VMCONN_RESPONSE_EDIT || id == VMCONN_RESPONSE_DELETE) {
        GtkTreeSelection *sel = gtk_tree_view_get_selection(
                                        GTK_TREE_VIEW(dlg->tv_conns));
        GtkTreeModel *model;
        GtkTreeIter iter;

        if (!sel)
            return;

        if (!gtk_tree_selection_get_selected(sel, &model, &iter))
            return;
        gtk_tree_model_get(model, &iter, VMCONN_COL_CONNDATA, &dlg->conn, -1);

        if (!dlg->conn)
            return;
    }

    switch(id) {
        case VMCONN_RESPONSE_ADD:
            _D("ask for a new connection");
            vnconn_edit_dialog(dlg);
            break;
        case VMCONN_RESPONSE_EDIT:
            _D("modify %s connection", dlg->conn->name);
            vnconn_edit_dialog(dlg);
            break;
        case VMCONN_RESPONSE_DELETE:
            vmconn_remove_conn(dlg, dlg->conn);
            vm_connection_enable(dlg->ctl, dlg->conn, FALSE);
            vm_connection_remove(dlg->ctl, dlg->conn);
            vm_connection_free(dlg->conn);
            dlg->conn = NULL;
            break;
        default:
            vmconn_dialog_quit(dlg);
            break;
    }
}

static void
vmconn_col_enabled_cb(GtkCellRendererToggle *render, gchar *path, gpointer data)
{
    VmconnDialog *dlg = (VmconnDialog *) data;
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(dlg->tv_conns));
    gboolean enabled;
    VmAppletConn *conn;
    GtkTreeIter iter;

    _F();

	gtk_tree_model_get_iter_from_string(model, &iter, path);
	gtk_tree_model_get(model, &iter,
					   VMCONN_COL_CONNDATA, &conn,
					   VMCONN_COL_ENABLED, &enabled,
					   -1);

    if (!conn)
       return;

    vm_connection_enable(dlg->ctl, conn, !enabled);
    vmconn_refresh_conn(dlg, conn);
}

static void
vmconn_append_conn(VmconnDialog *dlg, VmAppletConn *conn, gboolean active)
{
    GtkTreeIter iter;
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(dlg->tv_conns));

    _F();
    _D("appending: %s", conn->name);

    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
            VMCONN_COL_ENABLED, conn->enabled,
            VMCONN_COL_NAME, conn->name,
            VMCONN_COL_URI, conn->uri_str,
            VMCONN_COL_STATUS, conn->ptr ?
                    _("connected") : _("disconnected"),
            VMCONN_COL_CONNDATA,  conn,
            VMCONN_COL_DLGDATA, dlg,
            -1);

    if (active) {
        GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
        gtk_tree_view_set_cursor(GTK_TREE_VIEW(dlg->tv_conns),
                        path, NULL, FALSE);
    }
}

static void
vmconn_refresh_conn(VmconnDialog *dlg, VmAppletConn *conn)
{
    GtkTreeIter iter;
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(dlg->tv_conns));

    _F();

    if (!gtk_tree_model_get_iter_first(model, &iter))
        return;

    do {
        VmAppletConn *c = NULL;
        gtk_tree_model_get(model, &iter, VMCONN_COL_CONNDATA, &c, -1);

        if (c == conn) {
            _D("refreshing: %s", conn->name);
            gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                    VMCONN_COL_ENABLED, conn->enabled,
                    VMCONN_COL_NAME, conn->name,
                    VMCONN_COL_URI, conn->uri_str,
                    VMCONN_COL_STATUS, conn->ptr ?
                            _("connected") : _("disconnected"),
                    VMCONN_COL_CONNDATA,  conn,
                    VMCONN_COL_DLGDATA, dlg,
                    -1);
            break;
        }
    } while(gtk_tree_model_iter_next(model, &iter));
}


static void
vmconn_remove_conn(VmconnDialog *dlg, VmAppletConn *conn)
{
    GtkTreeIter iter;
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(dlg->tv_conns));

    _F();

    if (!gtk_tree_model_get_iter_first(model, &iter))
        return;

    do {
        VmAppletConn *c = NULL;
        gtk_tree_model_get(model, &iter, VMCONN_COL_CONNDATA, &c, -1);

        if (c == conn) {
            _D("deleting: %s", conn->name);
            gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
            break;
        }
    } while(gtk_tree_model_iter_next(model, &iter));
}

static GtkWidget *
vmconn_treeview_new(VmconnDialog *dlg)
{
    VmAppletConn      *conn;
    GtkWidget         *swin;
    GtkWidget         *tv;
    GtkListStore      *store;
    GtkTreeViewColumn *col;
    GtkCellRenderer   *render;

    _F();

    swin = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (swin),
            GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(swin),
            GTK_SHADOW_IN);

    /* creates treeview */
    tv = gtk_tree_view_new();
    gtk_container_add(GTK_CONTAINER(swin), tv);
    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tv), FALSE);
    gtk_tree_view_set_enable_search(GTK_TREE_VIEW(tv), FALSE);
    dlg->tv_conns = tv;

    /* creates data model */
    store = gtk_list_store_new(N_VMCONN_COLS,
                G_TYPE_BOOLEAN, G_TYPE_STRING,
                G_TYPE_STRING, G_TYPE_STRING,
                G_TYPE_POINTER, G_TYPE_POINTER);

    gtk_tree_view_set_model(GTK_TREE_VIEW(tv), GTK_TREE_MODEL(store));
    for (conn = dlg->ctl->conn; conn; conn = conn->next)
        vmconn_append_conn(dlg, conn, FALSE);

    /* Enabled */
    render =  gtk_cell_renderer_toggle_new();

    g_signal_connect(G_OBJECT(render), "toggled",
                G_CALLBACK(vmconn_col_enabled_cb), dlg);

    col = gtk_tree_view_column_new_with_attributes(
                _("enable"), render, "active", VMCONN_COL_ENABLED, NULL);
    gtk_tree_view_column_set_sizing(col, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tv), col);


    /* Name */
    render =  gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes(
                _("name"), render, "text", VMCONN_COL_NAME, NULL);
    gtk_tree_view_column_set_resizable(col, TRUE);
    gtk_tree_view_column_set_spacing (col, 5);
    gtk_tree_view_column_set_min_width (col, 30);
    gtk_tree_view_column_set_expand(col, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tv), col);

    /* URI */
    render =  gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes(
                _("uri"), render, "text", VMCONN_COL_URI, NULL);
    gtk_tree_view_column_set_resizable(col, TRUE);
    gtk_tree_view_column_set_spacing (col, 5);
    gtk_tree_view_column_set_min_width (col, 30);
    gtk_tree_view_column_set_expand(col, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tv), col);

    /* Status */
    render =  gtk_cell_renderer_text_new();
    col = gtk_tree_view_column_new_with_attributes(
                _("status"), render, "text", VMCONN_COL_STATUS, NULL);
    gtk_tree_view_column_set_sizing(col, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tv), col);

    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tv), TRUE);

    gtk_widget_show_all(swin);

    return swin;
}

void
vmconn_dialog(VmAppletCtl *ctl)
{
    GtkWidget *tv, *win;
    VmconnDialog *dlg;

    _F();

    dlg = g_new0(VmconnDialog, 1);

    win = gtk_dialog_new_with_buttons(_("VM Applet Connections"), NULL,
                            GTK_DIALOG_DESTROY_WITH_PARENT,
                            GTK_STOCK_ADD, VMCONN_RESPONSE_ADD,
                            GTK_STOCK_EDIT, VMCONN_RESPONSE_EDIT,
                            GTK_STOCK_DELETE, VMCONN_RESPONSE_DELETE,
                            GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                            NULL);
    dlg->d_conns = win;
    dlg->ctl = ctl;

    if (dlg->ctl->applet)
        gtk_window_set_screen(GTK_WINDOW(win),
                    gtk_widget_get_screen(ctl->applet));

    /* TODO: disable EDIT and DELETE buttons when no connection exists or is
     * not select */

    gtk_dialog_set_default_response(GTK_DIALOG(win), GTK_RESPONSE_CLOSE);
    gtk_dialog_set_has_separator(GTK_DIALOG(win), FALSE);
    gtk_container_set_border_width(GTK_CONTAINER(win), 12);
    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG (win)->vbox), VM_APPLET_BOX_SPACE);
    gtk_window_set_resizable(GTK_WINDOW(win), TRUE);
    gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER);
    gtk_window_set_default_size(GTK_WINDOW(win), 700, 300);
    gtk_window_set_decorated(GTK_WINDOW(win), TRUE);

    tv = vmconn_treeview_new(dlg);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(win)->vbox), tv, TRUE, TRUE, 0);

    g_signal_connect(G_OBJECT(win), "response",
            G_CALLBACK(vmconn_response_cb), dlg);

    gtk_widget_show_all(win);
}

/*
 * vim: set tabstop=4:
 * vim: set shiftwidth=4:
 * vim: set expandtab:
 */
