#include <stdio.h>
#include <stdlib.h>
#include <config.h>
#include <string.h>
#include <gtk/gtk.h>

#include "applet.h"

/*
 * Removes (but not deallocates) domain from list of domains
 */
VmAppletDomain *
vm_domainslist_rem(VmAppletDomain *list, VmAppletDomain *dom)
{
    VmAppletDomain *d = list;

    _F();

    if (!list)
        return NULL;

    if (dom==list)
        return list->next;

    while(d)
    {
        if (d->next==dom)
        {
            d->next = dom->next;
            dom->next = NULL;
            break;
        }
        d = d->next;
    }
    return list;
}

/*
 * Adds domain to the list of domains
 */
VmAppletDomain *
vm_domainslist_add(VmAppletDomain *list, VmAppletDomain *dom)
{
    VmAppletDomain *d = list;

    _F();

    dom->next = NULL;

    if (!list)
        return dom;

    while(d)
    {
        if (!d->next)
        {
            d->next = dom;
            break;
        }
        d = d->next;
    }
    return list;
}

/*
 * Looks up domain in the list of domains
 */
VmAppletDomain *
vm_domainslist_lookup(VmAppletDomain *list, char *uuid)
{
    _F();

    for (; list; list=list->next)
    {
        if (strcmp(list->uuid, uuid)==0)
            return list;
    }
    return NULL;
}

/*
 * Deallocates all domains in the list
 */
void
vm_domainslist_free(VmAppletDomain *list)
{
    VmAppletDomain *d = list;

    _F();

    while(d)
    {
        VmAppletDomain *next = d->next;
        vm_domain_free(d);
        d = next;
    }
}

/*
 * Sets real state of domain
 * -- we assume the domain that uses CPU is not blocked. We cannot
 *    use only the domain->state field, because it's alwais "blocked"
 *    on sigle CPU boxes where is just running this code.
 */
void
vm_domain_set_activity(VmAppletCtl *ctl, VmAppletDomain *dom)
{
    virDomainState old = dom->activity;

    _F();

    if (dom->state==VIR_DOMAIN_BLOCKED)
    {
        if ((ctl->delay * dom->cputime_ct) > VM_BLOCKED_TIME)
            dom->activity = VIR_DOMAIN_BLOCKED;
        else
            dom->activity = VIR_DOMAIN_RUNNING;
    }
    else if (dom->state==VIR_DOMAIN_NOSTATE)
    {
        /* better old state or running than nothing */
        dom->activity = old==VIR_DOMAIN_NOSTATE ? VIR_DOMAIN_RUNNING : old;
    }
    else
        dom->activity = dom->state;             /* other than running or blocked state */

    _D("domain-activity: %s: '%s'", dom->name, vm_domainstate_to_string(dom->activity));

    if (old != dom->activity)
    {
        _D("domain-changed: %s: activity", dom->name);
        dom->changed |= VM_DATA_ACTIVITY;
    }
}

/*
 * Allocates new domain struct and load info about the domain
 */
VmAppletDomain *
vm_domain_new(VmAppletCtl *ctl, virDomainPtr vir, char *uuid, VmAppletConn *conn)
{
    VmAppletDomain *dom;
    virDomainInfo info;
    char *p;

    _F();

    dom = g_new0(VmAppletDomain, 1);
    dom->ctl = ctl;
    dom->changed = 0;
    dom->conn = conn;

    dom->id = virDomainGetID(vir);
    dom->changed |= VM_DATA_ID;

    if (uuid)
        strcpy(dom->uuid, uuid);
    else
        virDomainGetUUIDString(vir, dom->uuid);
    dom->changed |= VM_DATA_UUID;

    dom->name = g_strdup(virDomainGetName(vir));
    dom->changed |= VM_DATA_NAME;

    _D("domain-new: %s: %s / %s", dom->conn->name, dom->name, dom->uuid);

    if ((p = virDomainGetOSType(vir)))
    {
        dom->ostype = g_strdup(p);
        dom->changed |= VM_DATA_OSTYPE;
        free(p);
    }
    if (virDomainGetInfo(vir, &info)==0)
    {
        dom->state = info.state;
        dom->changed |= VM_DATA_STATE;

        dom->maxmem = info.maxMem;
        dom->changed |= VM_DATA_MAXMEM;

        dom->usemem = info.memory;
        dom->changed |= VM_DATA_USEDMEM;

        dom->ncpus = info.nrVirtCpu;
        dom->changed |= VM_DATA_NCPUS;

        dom->cputime = info.cpuTime;
        dom->changed |= VM_DATA_CPUTIME;
    }

    vm_domain_set_activity(ctl, dom);
    return dom;
}

/*
 * Refreshs information about the domain
 */
void
vm_domain_refresh(VmAppletCtl *ctl, VmAppletDomain *dom, virDomainPtr vir, char *uuid)
{
    virDomainInfo info;
    const char *p;
    char xuuid[VM_UUID_SIZ];

    _F();

    _D("domain-refresh: %s : %s", dom->conn->name, dom->name);

    dom->changed = 0;

    if (dom->id!=virDomainGetID(vir))
    {
        _D("domain-changed: %s: id", dom->name);
        dom->id = virDomainGetID(vir);
        dom->changed |= VM_DATA_ID;
    }
    if (!uuid)
    {
        if (virDomainGetUUIDString(vir, xuuid)==0)
            uuid = xuuid;
    }
    if (uuid && strcmp(dom->uuid, uuid))
    {
        _D("domain-changed: %s: uuid", dom->name);
        dom->changed |= VM_DATA_UUID;
        strcpy(dom->uuid, uuid);
    }
    p = virDomainGetName(vir);
    if (p && strcmp(dom->name, p))
    {
        _D("domain-changed: %s: name", dom->name);
        g_free(dom->name);
        dom->name = g_strdup(p);
        dom->changed |= VM_DATA_NAME;
    }
    p = virDomainGetOSType(vir);
    if (p && strcmp(dom->ostype, p))
    {
        _D("domain-changed: %s: ostype", dom->name);
        g_free(dom->ostype);
        dom->ostype = g_strdup(p);
        dom->changed |= VM_DATA_OSTYPE;
    }
    if (p)
        free((void *)p);

    if (virDomainGetInfo(vir, &info)==0)
    {
        if (dom->state != info.state)
        {
            _D("domain-changed: %s: state", dom->name);
            dom->state = info.state;
            dom->changed |= VM_DATA_STATE;
        }
        if (dom->maxmem != info.maxMem)
        {
            _D("domain-changed: %s: maxmem", dom->name);
            dom->maxmem = info.maxMem;
            dom->changed |= VM_DATA_MAXMEM;
        }
        if (dom->usemem != info.memory)
        {
            _D("domain-changed: %s: usemem", dom->name);
            dom->usemem = info.memory;
            dom->changed |= VM_DATA_USEDMEM;
        }
        if (dom->ncpus != info.nrVirtCpu)
        {
            _D("domain-changed: %s: ncpus", dom->name);
            dom->ncpus = info.nrVirtCpu;
            dom->changed |= VM_DATA_NCPUS;
        }
        if (dom->cputime != info.cpuTime)
        {
            _D("domain-changed: %s: cputime", dom->name);
            dom->cputime = info.cpuTime;
            dom->changed |= VM_DATA_CPUTIME;
            dom->cputime_ct = 0;
        }
        else
            dom->cputime_ct++;
    }
    vm_domain_set_activity(ctl, dom);
}


static int
vm_domains_active_update_from_conn(VmAppletCtl *ctl, VmAppletConn *conn)
{
    VmAppletDomain *old, *new, *dom;
    int *ids, i, ndoms;
    int ct = 0;

    _F();

    ndoms = virConnectNumOfDomains(conn->ptr);
    if (ndoms <= 0)
    {
        /* remove all old active domains */
        dom = conn->dom;

        while(dom) {
            VmAppletDomain *next = dom->next;

            if (!dom->inactive) {
                conn->dom = vm_domainslist_rem(conn->dom, dom);
                vm_domain_free(dom);
            }
            dom = next;
        }
        return 0;
    }
    old = conn->dom;
    new = NULL;

    ids = g_malloc(sizeof(int) * ndoms);
    virConnectListDomains(conn->ptr, &ids[0], ndoms);

    for(i=0; i < ndoms; i++)
    {
        virDomainPtr vir = virDomainLookupByID(conn->ptr, ids[i]);
        char uuid[VM_UUID_SIZ];

        if (virDomainGetUUIDString(vir, uuid) < 0)
            continue;

        if ((dom = vm_domainslist_lookup(old, uuid)))
        {
            old = vm_domainslist_rem(old, dom);
            vm_domain_refresh(ctl, dom, vir, uuid);
        }
        else
            dom =  vm_domain_new(ctl, vir, uuid, conn);

        dom->inactive = FALSE;
        new = vm_domainslist_add(new, dom);
        virDomainFree(vir);
        ct++;
    }

    if (old)
        vm_domainslist_free(old);
    conn->dom = new;
    g_free(ids);

    return ct;
}

static int
vm_domains_inactive_update_from_conn(VmAppletCtl *ctl, VmAppletConn *conn)
{
    VmAppletDomain *old, *new, *dom;
    int i, ndoms;
    int ct = 0;
    char **names;

    _F();

    ndoms = virConnectNumOfDefinedDomains(conn->ptr);
    if (ndoms <= 0)
    {
        /* remove all old inactive domains */
        dom = conn->dom;

        while(dom) {
            VmAppletDomain *next = dom->next;

            if (dom->inactive) {
                conn->dom = vm_domainslist_rem(conn->dom, dom);
                vm_domain_free(dom);
            }
            dom = next;
        }
        return 0;
    }
    old = conn->dom;
    new = NULL;

    names = g_malloc(sizeof(char *) * ndoms);
    virConnectListDefinedDomains(conn->ptr, names, ndoms);

    for(i=0; i < ndoms; i++)
    {
        virDomainPtr vir = virDomainLookupByName(conn->ptr, names[i]);
        char uuid[VM_UUID_SIZ];

        free(names[i]);

        if (virDomainGetUUIDString(vir, uuid) < 0) {
            _D("Inactive domain hasn't UUID\n");
            continue;
        }

        if ((dom = vm_domainslist_lookup(old, uuid)))
        {
            old = vm_domainslist_rem(old, dom);
            vm_domain_refresh(ctl, dom, vir, uuid);
        }
        else
            dom =  vm_domain_new(ctl, vir, uuid, conn);

        dom->inactive = TRUE;
        new = vm_domainslist_add(new, dom);
        virDomainFree(vir);
        ct++;
    }

    if (old)
        vm_domainslist_free(old);
    conn->dom = new;
    g_free(names);

    return ct;
}

static int
vm_domains_update_from_conn(VmAppletCtl *ctl, VmAppletConn *conn)
{
    conn->ndoms = vm_domains_active_update_from_conn(ctl, conn);
    conn->ndoms += vm_domains_inactive_update_from_conn(ctl, conn);

    if (conn->ndoms == 0) {
        vm_domainslist_free(conn->dom);
        conn->dom = NULL;
    }

    return conn->ndoms;
}

/*
 * Updates all domains
 */
void
vm_domains_update(VmAppletCtl *ctl)
{
    VmAppletConn *conn = ctl->conn;

    _F();

    ctl->ndoms = 0;

    while(conn)
    {
        if (conn->enabled && conn->ptr)
            ctl->ndoms += vm_domains_update_from_conn(ctl, conn);
        conn = conn->next;
    }
}

/*
 * Deallocate the domain
 */
void
vm_domain_free(VmAppletDomain *dom)
{
    _F();

    if (!dom)
        return;

    if (dom->ctl->popmenu && dom->menuitem)
        gtk_widget_destroy(dom->menuitem);
    if (dom->name)
        g_free(dom->name);
    if (dom->ostype)
        g_free(dom->ostype);

    g_free(dom);
}


/*
 * Converts the domain state to string
 */
char *
vm_domainstate_to_string(virDomainState state)
{
    _F();
    switch (state) {
        case VIR_DOMAIN_BLOCKED:
            return _("blocked");
        case VIR_DOMAIN_RUNNING:
            return _("running");
        case VIR_DOMAIN_PAUSED:
            return _("paused");
        case VIR_DOMAIN_SHUTDOWN:
            return _("in shutdown");
        case VIR_DOMAIN_SHUTOFF:
            return _("shut off");
        default:
            return "unknown";    /* = dom0 state */
    }
    return NULL;
}

/*
 * Converts the domain state to icon stock Id
 */
char *
vm_domainstate_to_stockid(virDomainState state)
{
    _F();
    switch (state) {
        case VIR_DOMAIN_BLOCKED:
            return VM_STOCK_BLOCKED;
        case VIR_DOMAIN_RUNNING:
            return VM_STOCK_RUNNING;
        case VIR_DOMAIN_PAUSED:
            return VM_STOCK_PAUSED;
        case VIR_DOMAIN_SHUTDOWN:
            return VM_STOCK_SHUTDOWN;
        case VIR_DOMAIN_SHUTOFF:
            return VM_STOCK_SHUTOFF;
        default:
            return VM_STOCK_UNKNOWN;    /* = dom0 state */
    }
    return NULL;
}

/*
 * vim: set tabstop=4:
 * vim: set shiftwidth=4:
 * vim: set expandtab:
 */
