/*
 * =====================================================================================
 *
 *        Filename:  wrapper.c
 *
 *     Description:  Consolehelper wrapper
 *
 *         Version:  $Id$
 *         Created:  10.02.2006 10:56:18 CET
 *
 *          Author:  Karel Zak <kzak@redhat.com>
 *
 * =====================================================================================
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/types.h>
#include <libvirt/libvirt.h>

typedef struct VmWrapperCmd
{
    char   *name;
    int    (*doit)(virConnectPtr conn, virDomainPtr dom);
} VmWrapperCmd;

static int cmd_create(virConnectPtr conn, virDomainPtr dom);
static int cmd_shutdown(virConnectPtr conn, virDomainPtr dom);
static int cmd_destroy(virConnectPtr conn, virDomainPtr dom);
static int cmd_suspend(virConnectPtr conn, virDomainPtr dom);
static int cmd_resume(virConnectPtr conn, virDomainPtr dom);

VmWrapperCmd commands[] =
{
    { "create",     cmd_create },
    { "shutdown",   cmd_shutdown },
    { "destroy",    cmd_destroy },
    { "suspend",    cmd_suspend },   /* pause */
    { "resume",     cmd_resume },    /* unpause */
    { NULL, NULL }
};

static void
print_usage (char *command)
{
    VmWrapperCmd *cp;

    fprintf(stderr, "Usage: %s <command> <connURI> <domainUUID>\n\n"
                    " commands:\n", command);

    for (cp=commands; cp->name; cp++)
        fprintf(stderr, "   %s\n", cp->name);
	exit(EXIT_FAILURE);
}

static int
cmd_create(virConnectPtr conn, virDomainPtr dom)
{
    return virDomainCreate(dom)==0 ? 1 : 0;
}
static int
cmd_shutdown(virConnectPtr conn, virDomainPtr dom)
{
    return virDomainShutdown(dom)==0 ? 1 : 0;
}
static int
cmd_destroy(virConnectPtr conn, virDomainPtr dom)
{
    return virDomainDestroy(dom)==0 ? 1 : 0;
}
static int
cmd_suspend(virConnectPtr conn, virDomainPtr dom)
{
    return virDomainSuspend(dom)==0 ? 1 : 0;
}
static int
cmd_resume(virConnectPtr conn, virDomainPtr dom)
{
    return virDomainResume(dom)==0 ? 1 : 0;
}

int
main(int argc, char **argv)
{
    virDomainPtr domain = NULL;
    VmWrapperCmd *cmd = NULL, *cp;
    virConnectPtr conn = NULL;
    char *cmdstr, *uuid = NULL, *uri = NULL;
    int ret = 0;

    if (getuid()!=0)
    {
        fprintf(stderr, "%s: I work for root user only! Who are you?\n", argv[0]);
        exit(EXIT_FAILURE);
    }
    if (argc < 4)
        print_usage(argv[0]);

    cmdstr = argv[1];

    if (cmdstr==NULL || *cmdstr=='\0')
        exit(EXIT_FAILURE);

    for (cp=commands; cp->name; cp++)
    {
        if (strcasecmp(cp->name, cmdstr)==0)
        {
            cmd = cp;
            break;
        }
    }
    if (!cmd)
    {
        fprintf(stderr, "%s: unsupported command '%s'\n", argv[0], cmdstr);
        exit(EXIT_FAILURE);
    }

    uri = argv[2];
    uuid = argv[3];

    if (!(conn = virConnectOpen(uri)))
        fprintf(stderr, "%s: failed to connect to the hypervisor\n", argv[0]);
    else if (!(domain = virDomainLookupByUUIDString(conn, uuid)))
        fprintf(stderr, "%s: failed to get domain '%s'\n", argv[0], uuid);
    else
        ret = cmd->doit(conn, domain);

    if (domain)
        virDomainFree(domain);
    if (conn)
        virConnectClose(conn);

    exit(ret ? EXIT_SUCCESS : EXIT_FAILURE);
}

/*
 * vim: set tabstop=4:
 * vim: set shiftwidth=4:
 * vim: set expandtab:
 */
